from sqlalchemy import MetaData, create_engine, Table
from contextlib import contextmanager
from sqlalchemy.orm import sessionmaker, scoped_session
from dhuolib.config import logger
from sqlalchemy import text


class DatabaseConnection:
    def __init__(self, username, password, host, port, service_name):
        self.engine = self._get_engine(username, password, host, port, service_name)
        self.session = scoped_session(sessionmaker(bind=self.engine))

    def _get_engine(self, username, password, host, port, service_name):
        connection_string = f"oracle+cx_oracle://{username}:{password}@{host}:{port}/?service_name={service_name}"
        self.engine = create_engine(connection_string)
        return self.engine

    @contextmanager
    def session_scope(self, expire=False):
        self.session.expire_on_commit = expire
        try:
            yield self.session
            logger.info(f"Sessão foi iniciada {self.session}")
            self.session.commit()
        except Exception as e:
            self.session.rollback()
            logger.error(f"Erro na sessão {self.session}: {e}")
            raise
        finally:
            self.session.close()
            logger.info(f"Sessão foi finalizada {self.session}")


class GenericRepository:
    def __init__(self, data_base, table_name):
        self.db = data_base
        self.table_name = table_name
        self.metadata = MetaData()

    def load_table(self, session=None):
        return Table(self.table_name, self.metadata, autoload_with=session.bind)

    def create_table(self, list_columns: list):
        columns = [column for column in list_columns]
        schems = ", ".join(columns)
        table_schema = f"id SERIAL PRIMARY KEY, {schems}, predict_result VARCHAR(255), created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP, updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP"
        with self.db.session_scope() as session:
            query = text(f"CREATE TABLE IF NOT EXISTS {self.table_name} ({table_schema})")
            session.execute(query)

    def insert(self, table: dict):
        with self.db.session_scope() as session:
            session.execute(
                text(
                    f"INSERT INTO {self.table_name} ({', '.join(table.keys())}) VALUES ({', '.join([f':{key}' for key in table.keys()])})"),
                table,
            )
            inserted_predict = session.execute(text(
                f"SELECT * FROM {self.table_name} WHERE id = (SELECT MAX(id) FROM {self.table_name})")).fetchone()
        return inserted_predict

    def get_items_with_pagination(self, page, page_size):
        offset = (page - 1) * page_size
        with self.db.session_scope() as session:
            query = session.query(self.table).offset(offset).limit(page_size)
            items = query.all()
        return items

    def update(self, table: dict):
        with self.db.session_scope() as session:
            update_query = text(
                f"UPDATE {self.table_name} SET predict_result = :predict_result WHERE id = :id")
            session.execute(
                update_query, {'id': table['id'], 'predict_result': table['predict_result']})
            updated_predict = session.execute(text(
                f"SELECT * FROM {self.table_name} WHERE id = (SELECT MAX(id) FROM {self.table_name})")).fetchone()
        return updated_predict



# from dhuolib.clients.platform import DhuolibPlatformClient
# import sys
# import unittest
# from unittest.mock import patch, mock_open
# import base64
# from dhuolib.config import logger

# sys.path.append("src")


# class TestDhuolibPlatformClient(unittest.TestCase):
#     def setUp(self):
#         self.end_point = "http://localhost:8000"
#         self.dhuolib = DhuolibPlatformClient(service_endpoint=self.end_point, project_name="test-project6")
#         self.file_path = "tests/files/LogisticRegression_best.pickle"

#     def test_0_(self):
#         run_params = {
#             "experiment_id": "2",
#             "model_pkl_file": "tests/files/LogisticRegression_best.pickle",
#             "requirements_file": "tests/files/requirements.txt",
#         }
#         response = self.dhuolib.create_batch_project("test-project6")
#         response = self.dhuolib.batch_execute(cluster_size=1, script_filename="tests/files/script.py", requirements_filename="tests/files/requirements.txt")
#         response = self.dhuolib._create_cluster(1)
#         response = self.dhuolib._batch_run()

    # def test_4_transform_name_project_no_project_name(self):
    #     client = DhuolibClient(service_endpoint="http://example.com")
    #     self.assertIsNone(client.project_name)

    # def test_5_invalid_run_params(self):
    #     experiment_params = {
    #         "experiment_tags": {"version": "v1", "priority": "P1"},
    #     }
    #     response = self.dhuolib.create_experiment(experiment_params)
    #     self.assertEqual(list(response.keys()), ["error"])

    # @patch("requests.post")
    # def test_6_create_experiment_with_valid_params(self, mock_post):
    #     mock_response = mock_post.return_value
    #     mock_response.status_code = 201
    #     mock_response.json.return_value = {"experiment_id": "1"}

    #     experiment_params = {
    #         "experiment_name": "test_experiment",
    #         "experiment_tags": {"version": "v1", "priority": "P1"},
    #     }

    #     response = self.dhuolib.create_experiment(experiment_params)
    #     self.assertEqual(response, mock_response.json.return_value)

    # @patch("requests.post")
    # def test_7_run_experiment_with_valid_params(self, mock_post):
    #     mock_response = mock_post.return_value
    #     mock_response.status_code = 201
    #     mock_response.json.return_value = {
    #         "experiment_id": "experiment_id",
    #         "run_id": "run_id",
    #         "model_uri": "model_uri",
    #     }

    #     run_params = {
    #         "experiment_id": "2",
    #         "model_pkl_file": "tests/files/LogisticRegression_best.pickle",
    #         "requirements_file": "tests/files/requirements.txt",
    #     }
    #     response = self.dhuolib.run_experiment(run_params)

    #     self.assertEqual(response, mock_response.json.return_value)

    # @patch("requests.post")
    # def test_8_create_model_with_valid_params(self, mock_post):
    #     mock_response = mock_post.return_value
    #     mock_response.status_code = 201
    #     mock_response.json.return_value = {
    #         "current_stage": "Production",
    #         "last_updated_timestamp": 1713582060414,
    #         "model_version": "1",
    #         "run_id": "9434e517ed104958b6f5f47d33c79184",
    #         "status": "READY",
    #     }

    #     run_params = {
    #         "stage": "Production",
    #         "modelname": "nlp_framework",
    #         "modeltag": "v1",
    #         "run_id": "9434e517ed104958b6f5f47d33c79184",
    #         "requirements_file": "tests/files/requirements.txt",
    #         "model_uri": "model_uri",
    #     }

    #     response = self.dhuolib.create_model(run_params)

    #     self.assertEqual(response, mock_response.json.return_value)

    # @patch("requests.post")
    # def test_9_predict_online_with_valid_dataset(self, mock_post):
    #     mock_response = mock_post.return_value
    #     mock_response.status_code = 201
    #     mock_response.json.return_value = {
    #         "model_name": "nlp_framework",
    #         "predictions": [0, 1, 0, 1, 0, 1, 0, 1, 0, 1],
    #     }

    #     run_params = {
    #         "stage": "Production",
    #         "data": "tests/files/data_predict.csv",
    #         "modelname": "nlp_framework",
    #     }

    #     response = self.dhuolib.predict_online(run_params)

    #     self.assertEqual(response, mock_response.json.return_value)

    # @patch("requests.post")
    # def test_10_create_batch_project_successful(self, mock_post):
    #     project_name = "Test Project"
    #     mock_response = mock_post.return_value
    #     mock_response.status_code = 201
    #     mock_response.json.return_value = {"status": "success", "project_id": 123}
    #     response = self.dhuolib.create_batch_project(project_name)
    #     self.assertEqual(response, {"status": "success", "project_id": 123})

    # def test_11_create_batch_project_raises_exception_on_none(self):
    #     with self.assertRaises(ValueError) as context:
    #         self.dhuolib.create_batch_project(None)
    #         self.assertIn("project_name is required", str(context.exception))

    # def test_12_deploy_batch_project_no_project_name(self):
    #     self.dhuolib = DhuolibClient(service_endpoint=self.end_point)
    #     with self.assertRaises(ValueError) as context:
    #         self.dhuolib.deploy_batch_project("script.py", "requirements.txt")
    #         self.assertIn("Batch project is required", str(context.exception))

    # def test_13_deploy_batch_project_missing_files(self):
    #     with self.assertRaises(ValueError) as context:
    #         self.dhuolib.deploy_batch_project(None, None)
    #         self.assertIn(
    #             "script_filename and requirements_filename are required",
    #             str(context.exception),
    #         )

    # def test_14_deploy_batch_project_file_not_found(self):
    #     with patch("builtins.open", side_effect=FileNotFoundError("File not found")):
    #         response = self.dhuolib.deploy_batch_project(
    #             "script.py", "requirements.txt"
    #         )
    #         self.assertEqual(response, {"error": "File not found"})



# import unittest
# from sqlalchemy import create_engine, MetaData, Table, Column, Integer, String
# from sqlalchemy.orm import sessionmaker
# from dhuolib.repository import DatabaseConnection, GenericRepository  # Adjust this import according to your project structure
# from sqlalchemy import text


# class TestDatabaseConnection(unittest.TestCase):

#     @classmethod
#     def setUpClass(cls):
#         cls.connection_string = 'sqlite:///:memory:'
#         cls.db_connection = DatabaseConnection(cls.connection_string)

#     def test_database_connection_init(self):
#         db_connection = DatabaseConnection(self.connection_string)
#         self.assertIsNotNone(db_connection.engine)
#         self.assertIsNotNone(db_connection.session)

#     def test_session_scope(self):
#         with self.db_connection.session_scope() as session:
#             self.assertIsNotNone(session)


# class TestGenericRepository(unittest.TestCase):

#     @classmethod
#     def setUpClass(cls):
#         cls.connection_string = 'sqlite:///:memory:'
#         cls.db_connection = DatabaseConnection(cls.connection_string)
#         cls.table_name = 'test_table'
#         cls.repo = GenericRepository(cls.db_connection, cls.table_name)
#         cls.repo.create_table(['name VARCHAR(255)', 'age INTEGER'])

#     @classmethod
#     def tearDownClass(cls):
#         with cls.db_connection.session_scope() as session:
#             session.execute(text(f"DROP TABLE IF EXISTS {cls.table_name}"))

#     def setUp(self):
#         self.db_connection = self.__class__.db_connection
#         self.repo = self.__class__.repo
#         self.table_name = self.__class__.table_name

#     def tearDown(self):
#         with self.db_connection.session_scope() as session:
#             session.execute(text(f"DELETE FROM {self.table_name}"))

#     def test_create_table(self):
#         new_repo = GenericRepository(self.db_connection, 'new_table')
#         new_repo.create_table(['description TEXT'])
#         with self.db_connection.session_scope() as session:
#             result = session.execute(text("SELECT name FROM sqlite_master WHERE type='table' AND name='new_table';")).fetchone()
#             self.assertIsNotNone(result)
#             self.assertEqual(result[0], 'new_table')

#     def test_insert(self):
#         data = {'id': 1, 'name': 'John Doe', 'age': 30}
#         inserted = self.repo.insert(data)
#         with self.repo.db.session_scope() as session:
#             result = session.execute(text(f"SELECT * FROM {self.repo.table_name} WHERE id=1")).fetchone()
#             self.assertIsNotNone(result)
#             self.assertEqual(result.name, 'John Doe')
#             self.assertEqual(result.age, 30)
#             self.assertEqual(inserted.name, 'John Doe')
#             self.assertEqual(inserted.age, 30)

#     def test_get_items_with_pagination(self):
#         for i in range(10):
#             data = {'id': i+1, 'name': f'User {i+1}', 'age': 20 + i}
#             self.repo.insert(data)
        
#         items = self.repo.get_items_with_pagination(1, 5)
#         self.assertEqual(len(items), 5)
#         self.assertEqual(items[0].name, 'User 1')
#         self.assertEqual(items[4].name, 'User 5')

#         items = self.repo.get_items_with_pagination(2, 5)
#         self.assertEqual(len(items), 5)
#         self.assertEqual(items[0].name, 'User 6')
#         self.assertEqual(items[4].name, 'User 10')

#     def test_update(self):
#         data = {'id': 1, 'name': 'John Doe', 'age': 40, 'predict_result': 'Negative'}
#         self.repo.insert(data)
#         updated_data = {'id': 1, 'name': 'John Doe', 'age': 40, 'predict_result': 'Positive'}
#         updated = self.repo.update(updated_data)
#         with self.repo.db.session_scope() as session:
#             result = session.execute(text(f"SELECT * FROM {self.repo.table_name} WHERE id=1")).fetchone()
#             self.assertIsNotNone(result)
#             self.assertEqual(data["predict_result"], 'Negative')
#             self.assertEqual(updated.predict_result, 'Positive')

#     def test_insert_invalid_data(self):
#         invalid_data = {'id': 1, 'invalid_column': 'Invalid Data'}
#         with self.assertRaises(Exception):
#             self.repo.insert(invalid_data)



from contextlib import contextmanager
from sqlalchemy import create_engine, MetaData
from sqlalchemy.orm import sessionmaker, scoped_session
from src.dhuolib.config import logger


class DatabaseConnection:
    def __init__(self, connection_string):
        self.engine = create_engine(connection_string)
        self.metadata = MetaData(bind=self.engine)
        self.Session = scoped_session(sessionmaker(bind=self.engine))

    def get_session(self):
        return self.Session()

    @contextmanager
    def session_scope(self, expire=False):
        session = self.get_session()
        session.expire_on_commit = expire
        try:
            yield session
            logger.info(f"Sessão foi iniciada {session}")
            session.commit()
        except Exception as e:
            session.rollback()
            logger.error(f"Erro na sessão {session}: {e}")
            raise
        finally:
            session.close()
            logger.info(f"Sessão foi finalizada {session}")





def validade_name(string: str):
    if string is None:
        raise ValueError("string must be informed!")
    if len(string) < 5:
        raise ValueError("string must have at least 5 characters!")
    if len(string) > 30:
        raise ValueError(
            "string cannot be longer than 30 characters!"
        )
    # special characters not allowed
    special_characters = "\"'!@#$%¨&*()+=§`[]{}ªº;.,~^/?\\|áéíóúâêîôûãõäëïöüñç "
    string = string.lower()
    if any(c in special_characters for c in string):
        raise ValueError(
            "string has invalid characters. Only '-' and '_' are allowed!"
        )
    # start and end
    if string.startswith("_"):
        raise ValueError("string can't start with _!")
    if string.startswith("-"):
        raise ValueError("string can't start with -!")
    if string.endswith("_"):
        raise ValueError("string can't end with _!")
    if string.endswith("-"):
        raise ValueError("string can't end with -!")
    
    return string
